package loja.ejb;

import javax.ejb.*;
import java.util.*;
import java.math.BigDecimal;

import loja.integration.*;
import loja.*;

/**
 * Este bean nao deve ter interfaces Remote porque eh composto
 * por objetos locais (ItemLocal) que clientes remotos nao podem ter acesso.
 */
public class CompraBean implements EntityBean {
	
	protected EntityContext ctx;
	
	private String numero;
	private String status;
	private List itens; // colecao de refs. para EJBLocalObjects
	
	private ItemLocalHome itemHome;
	
	private CompraDAO compraDAO;
	
	//////////////////// life-cycle ////////////////////////////
	
	/////////// 1. Context /////////////
	
	public void setEntityContext(EntityContext ctx) {
		System.out.println("CompraBean: setEntityContext()");
		this.ctx = ctx;
		try {
			ServiceLocator locator = ServiceLocator.getInstance();
			itemHome = (ItemLocalHome)locator.findLocalEJB("ejb/loja/item");
		} catch (LojaException e) {
			throw new EJBException("Objeto de relacionamento nao encontrado: " + e);
		}
	}
	
	public void unsetEntityContext() {
		ctx = null;
	}

	/////////// 2. Home ///////////////
	
	public void  ejbPostCreate(String numero) {}
	public CompraPK ejbCreate(String numero) throws CreateException {
		System.out.println("CompraBean: ejbCreate("+numero+")");
		this.numero = numero;
		this.status = "Iniciado";
		itens = new ArrayList();
		try {
			compraDAO = JdbcCompraDAO.getInstance();
			CompraVO compra = new CompraVO(numero, status);
			return new CompraPK(compraDAO.create(compra));
		} catch (LojaException e) {
			throw new CreateException("Incapaz de obter instancia DAO: " + e);
		}
	}

	public CompraPK ejbFindByPrimaryKey(CompraPK pk) throws FinderException {
		System.out.println("CompraBean: ejbFindByPrimaryKey()");
		try {
			compraDAO = JdbcCompraDAO.getInstance();
			if (compraDAO.exists(pk.numero)) {
				return pk;
			} else {
				throw new ObjectNotFoundException("Objeto " + pk.numero + " nao existe!");
			}
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
	}
	
	public Collection ejbFindAll() throws FinderException {
		System.out.println("CompraBean: ejbFindAll()");
		String[] numeros = null;
		try {
			compraDAO = JdbcCompraDAO.getInstance();
			numeros = compraDAO.getAll();
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
		List list = new ArrayList(numeros.length);
		for (int i = 0; i < numeros.length; i++) {
			list.add(new CompraPK(numeros[i]));
		}
		return list;
	}
	
	public Collection ejbFindByStatus(String status) throws FinderException {
		System.out.println("CompraBean: ejbFindByStatus("+status+")");
		String[] numeros = null;
		try {
			compraDAO = JdbcCompraDAO.getInstance();
			numeros = compraDAO.getByStatus(status);
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
		List list = new ArrayList(numeros.length);
		for (int i = 0; i < numeros.length; i++) {
			list.add(new CompraPK(numeros[i]));
		}
		return list;
	}
	
	/**
	 * Este mtodo pode ser implementado de duas formas: 1) criando um metodo no DAO que
	 * faca o query composto nas tabelas Compra e Item para descobrir o total, ou 2)
	 * usando os objetos locais (implementacao escolhida abaixo). 
	 */
	public Collection ejbFindByTotal(BigDecimal min, BigDecimal max) throws FinderException {
		System.out.println("CompraBean: ejbFindByTotal("+min+","+max+")");
		String[] todosOsNumeros = null;
		List numerosSelecionados = new ArrayList();
		try {
			compraDAO = JdbcCompraDAO.getInstance();
			todosOsNumeros = compraDAO.getAll();
			for (int i = 0; i < todosOsNumeros.length; i++) {
				CompraLocalHome home = (CompraLocalHome)ctx.getEJBLocalHome();
				CompraLocal compra = home.findByPrimaryKey(new CompraPK(todosOsNumeros[i]));
				BigDecimal total = compra.getTotal();
				if (total.compareTo(min) >= 0 && total.compareTo(max) <= 0) {
					numerosSelecionados.add(new CompraPK(todosOsNumeros[i]));
				}
			}
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
		return numerosSelecionados;
	}
	
	/////////// 3. Persistence and Container Call-back ///////////////////
	
	public void ejbLoad() {
		System.out.println("CompraBean: ejbLoad()");
		CompraPK compraPK = (CompraPK)ctx.getPrimaryKey(); 
		CompraVO compra = null;
		try {
			compraDAO = JdbcCompraDAO.getInstance();
			compra = compraDAO.load(compraPK.numero);
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
		numero 		= compra.getNumero();
		status      = compra.getStatus();
		
		// Localizar itens associados a esta compra
		try {
			itens = (List) itemHome.findByCompra(compraPK.numero);
		} catch (FinderException e) {
			throw new EJBException("Objeto de relacionamento nao encontrado: " + e);
		}
	}
	
	public void ejbStore() {
		System.out.println("CompraBean: ejbStore()");
		CompraVO compra = new CompraVO(numero, status);
		// Nao  preciso guardar informaes de itens. A criao 
		// de cada item ja contem como FK o PK da compra
		try {
			compraDAO = JdbcCompraDAO.getInstance();
			compraDAO.store(compra);
		} catch (LojaException e) {
			throw new EJBException("Falha na comunicacao com o banco: " + e);
		}
	}
	
	public void ejbRemove() {
		System.out.println("CompraBean: ejbRemove()");
		CompraPK pk = (CompraPK)ctx.getPrimaryKey();
		// Cascade delete para itens de compra
		try {
			Iterator it = itens.iterator();
			while(it.hasNext()) {
				ItemLocal item = (ItemLocal) it.next();
				item.remove();
			}
		} catch (RemoveException e) {
			throw new EJBException("CompraBean: Cascade delete falhou: " + e);
		}
		
		// agora falta remover a compra
		try {
			compraDAO = JdbcCompraDAO.getInstance();
			compraDAO.remove(pk.numero);
		} catch (LojaException e) {
			throw new EJBException("CompraBean: Falha na comunicacao com o banco - ejbRemove(): " + e);
		}
	}
	
	public String ejbHomeGetProximoNumero() {
		try {
			compraDAO = JdbcCompraDAO.getInstance();
			return compraDAO.criarNumero();
		} catch (LojaException e) {
			throw new EJBException("Incapaz de criar numero: " + e);
		}
	}
	
	public void ejbActivate() {}
	public void ejbPassivate() {}
	
	
	//////////// BUSINESS AND ACESSOR ///////////////
	
	public String getNumero() {
		System.out.println("CompraBean: getNumero()");
		return numero;
	}
	public void setNumero(String numero) {
		System.out.println("CompraBean: setUserId("+numero+")");
		this.numero = numero;
	}
	
	public String getStatus() {
		System.out.println("CompraBean: getStatus()");
		return status;
	}
	public void setStatus(String status) {
		System.out.println("CompraBean: setStatus("+status+")");
		this.status = status;
	}
	
	public ItemLocal[] getItens() {
		ItemLocal[] buffer = new ItemLocal[itens.size()];
		return (ItemLocal[]) itens.toArray(buffer);
	}
	
	public void setItens(ItemLocal[] itensArray) {
		itens.clear();
		for (int i = 0; i < itensArray.length; i++) {
			itens.add(itensArray[i]);
		}
	}
	
	public void addItem(ItemLocal item) {
		System.out.println("CompraBean: addItem("+item+")");
		itens.add(item);
	}
	
	public void removeItem(ItemLocal item) {
		System.out.println("CompraBean: removeItem("+item+")");
		itens.remove(item);
	}
	
	public BigDecimal getTotal() {
		System.out.println("CompraBean: getTotal() para " + itens.size() + " itens.");
		BigDecimal total = new BigDecimal(0.0);
		Iterator it = itens.iterator();
		while (it.hasNext()) {
			ItemLocal item = (ItemLocal)it.next();
			total = total.add(item.getSubTotal());
		}
		return total;
	}

}
